@@ -1,12 +1,16 @@
+// +build amd64
+
 package importer
 
 import (
+	"bufio"
 	"bytes"
 	"context"
 	"crypto/md5"
 	"errors"
 	"net/url"
 	"os"
+	"strings"
 
 	libnbd "github.com/mrnold/go-libnbd"
 	. "github.com/onsi/ginkgo"
@@ -351,6 +355,60 @@ var _ = Describe("VDDK data source", func() {
 		mockTerminationChannel <- os.Interrupt
 		Expect(err).ToNot(HaveOccurred())
 	})
+
+	It("should reduce pread length for vCenter endpoints", func() {
+		newVddkDataSource = createVddkDataSource
+		diskName := "testdisk.vmdk"
+
+		currentVMwareFunctions.Properties = func(ctx context.Context, ref types.ManagedObjectReference, property []string, result interface{}) error {
+			switch out := result.(type) {
+			case *mo.VirtualMachine:
+				if property[0] == "config.hardware.device" {
+					out.Config = createVirtualDiskConfig(diskName, 12345)
+				}
+			}
+			return nil
+		}
+		_, err := NewVDDKDataSource("http://esx.test", "user", "pass", "aa:bb:cc:dd", "1-2-3-4", diskName, "", "", "", v1.PersistentVolumeFilesystem)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(MaxPreadLength).To(Equal(uint32(MaxPreadLengthESX)))
+		_, err = NewVDDKDataSource("http://vcenter.test", "user", "pass", "aa:bb:cc:dd", "1-2-3-4", diskName, "", "", "", v1.PersistentVolumeFilesystem)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(MaxPreadLength).To(Equal(uint32(MaxPreadLengthVC)))
+	})
+})
+
+var _ = Describe("VDDK log watcher", func() {
+	DescribeTable("should set VDDK connection information from nbdkit log stream, ", func(messages []string, results []string) {
+		vddkVersion = "not set"
+		vddkHost = "not set"
+		logbuffer := new(bytes.Buffer)
+		logreader := bufio.NewReader(logbuffer)
+		watcher := createNbdKitLogWatcher()
+		for _, message := range messages {
+			logbuffer.WriteString(message + "\n")
+		}
+		watcher.Start(logreader)
+		watcher.Stop()
+		Expect(vddkVersion).To(Equal(results[0]))
+		Expect(vddkHost).To(Equal(results[1]))
+	},
+		Entry("with no data", []string{}, []string{"not set", "not set"}),
+		Entry("with good version string", []string{"nbdkit: debug: VMware VixDiskLib (7.0.0) Release build-15832853"}, []string{"7.0.0", "not set"}),
+		Entry("with alternate good version string", []string{"nbdkit: debug: VMware VixDiskLib (7.0) Release build-15832853"}, []string{"7.0", "not set"}),
+		Entry("with bad version string", []string{"nbdkit: debug: VMware VixDiskLib 7.0 Release build-15832853"}, []string{"not set", "not set"}),
+		Entry("with good host string", []string{"nbdkit: vddk[1]: debug: DISKLIB-LINK  : Opened 'vpxa-nfcssl://[iSCSI_Datastore] test/test.vmdk@esx15.test.lan:902' (0xa): custom, 50331648 sectors / 24 GB."}, []string{"not set", "esx15.test.lan"}),
+		Entry("with alternate good host string", []string{"nbdkit: vddk[1]: debug: DISKLIB-LINK  : Opened 'http://disk@esx:1234' (0xa): custom, 50331648 sectors / 24 GB."}, []string{"not set", "esx"}),
+		Entry("with bad host string", []string{"nbdkit: vddk[1]: debug: DISKLIB-LINK  : Opened 'vpxa-nfcssl://esx' (0xa): custom, 50331648 sectors / 24 GB."}, []string{"not set", "not set"}),
+		Entry("with good version and host strings", []string{
+			"nbdkit: debug: VMware VixDiskLib (7.0.0) Release build-15832853",
+			"nbdkit: vddk[1]: debug: DISKLIB-LINK  : Opened 'vpxa-nfcssl://disk@esx:1234' (0xa): custom, 50331648 sectors / 24 GB."},
+			[]string{"7.0.0", "esx"}),
+		Entry("with bad version and host strings", []string{
+			"nbdkit: debug: VMware VixDiskLib ",
+			"nbdkit: vddk[1]: debug: DISKLIB-LINK  : Opened '"},
+			[]string{"not set", "not set"}),
+	)
 })
 
 type mockNbdOperations struct{}
@@ -433,12 +491,18 @@ func createMockVddkDataSink(destinationFile string, size uint64, volumeMode v1.P
 	return sink, nil
 }
 
-type mockVMwareConnectionOperations struct{}
+type mockVMwareConnectionOperations struct {
+	Endpoint string
+}
 
 func (ops *mockVMwareConnectionOperations) Logout(context.Context) error {
 	return nil
 }
 
+func (ops *mockVMwareConnectionOperations) IsVC() bool {
+	return strings.Contains(ops.Endpoint, "vcenter")
+}
+
 type mockVMwareFunctions struct {
 	Properties            func(context.Context, types.ManagedObjectReference, []string, interface{}) error
 	Reference             func() types.ManagedObjectReference
@@ -488,7 +552,7 @@ func createMockVMwareClient(endpoint string, accessKey string, secKey string, th
 	ctx, cancel := context.WithCancel(context.Background())
 
 	return &VMwareClient{
-		conn:       &mockVMwareConnectionOperations{},
+		conn:       &mockVMwareConnectionOperations{endpoint},
 		cancel:     cancel,
 		context:    ctx,
 		moref:      "vm-1",
